
if (!window.widget)
{
  // flow comes here when emulater loaded as a web page
  // assign something to Control so that default_devices.js doesn't throw an error
  window.Control = {};
}
else
{
  window.Control = {
    
    devices: widget.preferenceForKey('devices'),
    devicesDefault: null, // set by index.html
    
    lastDevice: widget.preferenceForKey('lastDevice'),
    lastWidget: widget.preferenceForKey('lastWidget'),
    
    preferences: widget.preferenceForKey('preferences') || {},
    
    config : {
      showOutline: widget.preferenceForKey('config-showOutline') !='0',
      animations:  widget.preferenceForKey('config-animations')  !='0'
    },
    
    init: function()
    {
      if (!this.devices)
      {
        this.loadDefaultDevices();
      }
      
      scrollbars.init();
      ui.init();
      fileio.init();
      editing.init();
      deviceSelector.init();
      manager.init();
      messenger.init();
      pluginHandler.init(function()
      {
        ui.createPluginsEditingList();
        pluginPanel.init();
        
        Control.prepareWidgetPreferences();
        
        Control.loadLastDevice()
        
      }); // supply a callback
    },
    
    device: { // units are all integer pixels unless specified
      settings: null, // becomes a copy of the device definition object
      state: {
        loaded: false,
        chrome: null,
        dock: null,
        rotated: false
      },
      media: null, // an instance of Media()
      screen: {
        /* width, height, availWidth, availHeight */
      },
      widget: {
        /*
        width,
        height,
        top,
        left,
        opened (boolean: step1 - if widget iframe has been added to dom),
        began (boolean: step2 - once the index.html file loads),
        loaded (boolean: step3 - if widget's onload event has fired),
        preferences (object),
        storage (int bytes used),
        mode,
        defaultMode,
        dockable,
        dockX,
        dockY,
        configXML
        */
      },
      plugins: []
    },
    
    /**
     * Goes through widget preferences and removes any no longer needed.
     */
    
    prepareWidgetPreferences: function()
    {
      // remove preferences of old widgets
      var culled = false;
      var j;
      for (var i in this.preferences)
      {
        if (!manager.getWidgetByLocationPath(i))
        {
          delete this.preferences[i];
          culled = true;
        }
      }
      
      culled && this.savePreferences();  
    },
    
    loadDefaultDevices: function()
    {
      this.devices = this.devicesDefault.concat();
      for (var i=0; i<this.devices.length; i++)
      {
        this.devices[i].id = i+''; // need unique id strings
      }
    },
    
    savePreferences: function()
    {
      if (this.device.widget.locationPath)
      {
        this.preferences[this.device.widget.locationPath] = this.device.widget.preferences;
      }
      
      widget.setPreferenceForKey(this.preferences,'preferences');
    },
    
    getCurrentDevice : function()
    {
      var settings = null;
      for (var i=0; i<this.devices.length; i++)
      {
        if (this.devices[i].id == this.lastDevice) { break; }
      }
      return (i==this.devices.length) ? null : this.devices[i];
    },
    
    getIndex : function()
    {
      for (var i=0; i<this.devices.length; i++)
      {
        if (this.devices[i].id == this.lastDevice)
        {
          return i;
        }
      }
      throw new Error('Unknown device index');
    },
    
    getDeviceById : function(id)
    {
      var index = this.getDeviceIndexById(id);
      return id < 0 ? null : this.devices[index];
    },
    
    getDeviceIndexById : function(id)
    {
      for (var i=0; i<this.devices.length; i++)
      {
        if (this.devices[i].id == id)
        {
          return i;
        }
      }
      return -1;
    },
    
    /**
     * Deletes a plugin.
     * Removes plugin from any device's plugin list.
     * Calss necessary methods to completely remove the plugin, both from
     * storage and from places in UI.
     * 
     * @param pluginName {String} The name of the plugin to remove.
     * @param keepReferences {Boolean} If true then devices using this plugin will still reference it (used when replacing plugins)
     *
     */
    
    deletePlugin: function(pluginName, keepReferences)
    {
      var i = 0;
      var index = 0;
      
      fileio.deletePlugin(pluginName);
      
      if (!keepReferences)
      {
        for (var i=0; i<Control.devices.length; i++)
        {
          index = Control.devices[i].plugins.indexOf(pluginName)
          if (index >= 0)
          {
            Control.devices[i].plugins.splice(index, 1);
            Control.saveDevice(Control.devices[i]);
          }
        }
      }
      pluginPanel.removePluginUI(pluginName);
      pluginHandler.deletePlugin(pluginName);
      ui.createPluginsEditingList();
    },
    
    addPlugin: function(path)
    {
      pluginHandler.preparePlugin(path, function(plugin)
      {
        pluginPanel.addPluginUI(plugin);
        ui.createPluginsEditingList();
      });
    },
    
    loadDevice : function(device)
    {
      var self = this;
      
      emulator.closeWidget();
      ui.loadDevice(device, function()
      {
        emulator.loadDevice(device);
        editing.loadDevice(device);
        deviceSelector.update();
        
        if (Control.lastWidget)
        {
          Control.loadWidget(Control.lastWidget);
        }
        else
        {
          g('desktop').style.display = 'block';
        }
      });
      
      this.lastDevice = device.id;
      widget.setPreferenceForKey(this.lastDevice, 'lastDevice'); // remember last device
    },
    
    /**
     * Will load up whichever device was used last.
     */
    
    loadLastDevice : function()
    {
      if (!this.devices.length) { return; }
      
      // load last device used
      for (var i=0; i<this.devices.length; i++)
      {
        if (this.devices[i].id == this.lastDevice)
        {
          break;
        }
      }
      if (i==this.devices.length)
      {
        // no device ever used, so look for first device to have selected:true
        for (var i=0; i<this.devices.length; i++)
        {
          if (this.devices[i].selected)
          {
            break;
          }
        }
        
        // no device has selected:true, so use the first in the list
        if (i==this.devices.length)
        {
          i = 0;
        }
        
        // record as last device
        this.lastDevice = this.devices[i].id;
      }
      
      this.loadDevice(this.devices[i]);
    },
    
    rotateDevice: function()
    {
      var d = this.device;
      
      if (d.settings.rotatable)
      {
        d.state.rotated = !d.state.rotated;
        d.state.chrome = d.state.rotated && d.settings.rotatable.chrome
                         || d.settings.chrome || [0,0,0,0];
        d.state.dock = d.state.rotated && d.settings.rotatable.dock
                        || d.settings.dock || null;
        
        emulator.rotate();
      }
    },
    
    /**
     * Checks existing devices if there is already one just like it.
     * Checks all properties for equality.
     *
     * @param settings {Object} Device description
     * @returns {Boolean} True if the device is unique, else false
     */
    
    isDeviceNew: function(settings)
    {            
      for (var i=0; i<this.devices.length; i++)
      {
        if (areObjectsEqual(settings, this.devices[i], ['id']))
        {
          return false;
        }
      }
      return true;
    },
    
    saveDevice: function(settings)
    {
      var index = this.getDeviceIndexById(settings.id);
      
      if (index < 0)
      {
        settings.id = 'random'+Math.random();
        this.devices.push(settings);
        editing.addDevice(settings);
      }
      else
      {
        this.devices[index] = settings;
        editing.updateDevice(settings);
      }
      
      widget.setPreferenceForKey(this.devices, 'devices');
      
      if (this.lastDevice == settings.id)
      {
        this.loadDevice(settings);
      }
      else
      {
        deviceSelector.update();
      }
    },
    
    deleteDevice: function(settings)
    {
      // find the device to delete
      for (var i=0; i<this.devices.length; i++)
      {
        if( this.devices[i] == settings)
        {
          break;
        }
      }
      
      if (i==this.devices.length) { return; }
      
      if (this.devices.length<2)
      {
        // restore defaults when nothing left
        this.loadDefaultDevices();
        
        editing.removeDevice(i);
        
        editing.generateDevices();
        this.loadDevice(this.devices[0]);
      }
      else
      {
        this.devices.splice(i,1);
        editing.removeDevice(i);
        
        if (settings.id == this.lastDevice)
        {
          this.loadDevice(this.devices[Math.min(this.devices.length-1, i)]);
        }
        else
        {
          editing.focusDevice(this.devices[Math.min(this.devices.length-1, i)]);
          deviceSelector.update();
        }
      }
      
      widget.setPreferenceForKey(this.devices,'devices');
    },
    
    /**
     * Imports devices
     *
     * @param obj {Object} The object found in the device definition json file.
     */
    
    importDevices: function(obj)
    {
      if (!obj instanceof Array)
      {
        // don't want to make special cases for when there's only one device
        // so force it to be an array
        obj = [obj];
      }
      
      var notAdded = 0;
      
      for (var i = 0; i<obj.length; i++)
      {
        if (Control.isDeviceNew(obj[i]))
        {
          Control.saveDevice(obj[i]);
        }
        else
        {
          notAdded++;
        }
      }
      
      if (notAdded)
      {
        ui.alert('Duplicate devices', notAdded + ' device(s) were found in your device list so were not imported.');
      }
    },
    
    /**
     * Callback function for opera.io.filesystem.browseForDirectory
     * Validates the directory as a widget
     * If it is valid then it stores it
     * 
     * @param locationPath {String} The widget's real path (from mountpoint)
     */
    
    loadWidget: function(locationPath)
    {
      if (this.device.widget.opened)
      {
        this.closeWidget();
      }
      
      // load preferences
      this.device.widget.preferences = this.preferences[locationPath] || {};
      
      var launchPath = fileio.prepareWidgetForLaunch(locationPath);
      
      this.lastWidget = locationPath;
      
      if (!launchPath)
      {
        Control.failWidget('index');
        return;
      }
      
      widget.setPreferenceForKey(this.lastWidget,'lastWidget');
      
      emulator.loadWidget(launchPath, locationPath);
    },
    
    failWidget: function(type)
    {
      widget.setPreferenceForKey(null,'lastWidget');
      
      ui.failWidget(type);
    },
    
    closeWidget: function()
    {
      if (!this.device.widget.opened) { return; } // not opened
      
      this.device.widget.opened = false;
      this.device.widget.began = false;
      emulator.closeWidget();
      ui.closeWidget();
      
      // forget as last widget
      Control.lastWidget = null;
      widget.setPreferenceForKey(null,'lastWidget');
    },
    
    startPlugins: function(frameWindow)
    {
      this.device.widget.JSPlugins = emulator.detectJSPlugins(this.device.widget.configXML);
      
      var plugins = this.device.plugins;
      
      for (var i=0; i<plugins.length; i++)
      {
        try {
          ui.showPlugin(plugins[i], pluginHandler.startPlugin(plugins[i], frameWindow));
        }
        catch(e)
        {
          opera.postError(e);
        }
      }
    }
      
  };
}