document.addEventListener('DOMContentLoaded', function()
{
  if (window.widget) 
  {
    crawl();
  }
  else
  {
    document.body.innerHTML = '\
      <div style="font-size:12pt;margin:12pt;">\
        <p style="font-weight: bold;">Widget Emulator<\/p>\
        <p>This document is inteded to be run as a widget,\
        drag the file <code>config.xml</code> into the Opera window to continue.\
        <p>View the <kbd>docs/articles/widget_emulator.html</kbd> file in the SDK package\
        for more information.<\/p>\
      <\/div>\
      ';
  }
}, false);

window.addEventListener('resize', function(evt)
{
  var iframe = document.getElementsByTagName('iframe')[0];
  
  var pos = [window.screenLeft, window.screenTop];
  
  if (iframe)
  {
    iframe.style.width = document.documentElement.clientWidth + 'px';;
    iframe.style.height = document.documentElement.clientHeight + 'px';
    
    // move the window if it is suitable to:
    
    if (pos[0] + document.documentElement.clientWidth > screen.availWidth &&
        pos[0] > 0)
    {
      pos[0] = Math.max(0, screen.availWidth - document.documentElement.clientWidth);
    }
    
    if (pos[1] + document.documentElement.clientHeight > screen.availHeight &&
        pos[1] > 0)
    {
      pos[1] = Math.max(0, screen.availHeight - document.documentElement.clientHeight);
    }
    
    if (pos[0] != window.screenLeft || pos[1] != window.screenTop)
    {
      window.moveTo(pos[0], pos[1]);
    }
  }
  
  
  
  
  
}, true);

var messenger = {  
  /**
   * Takes an object, turns it into a JSON string and posts it to the parent.
   * 
   * @param data {Object} The data object to send, should have a 'command' property
   */
  
  postMessage: function(data)
  {
    document.frames[0].postMessage(JSON.stringify(data));
  },
  
  alert: function(title, text)
  {
    messenger.postMessage({
      command: "alert",
      title: title,
      text: text
    });
  }
}

var receiver = {
  
  listener: function(event)
  {
    var data = JSON.parse(event.data);
    receiver[data.command](data);
  },
  
  close: function(data)
  {
    window.close();
  },
  
  browseForWidget: function()
  {
    showPopup('browseForWidget', fileio.browseForWidget);
  }
};

window.addEventListener("message", receiver.listener, false);

function showPopup(text, callback)
{
  var div = document.createElement('div');
  div.textContent = text;
  
  var button = document.createElement('button');
  button.textContent = 'Ok';
  button.addEventListener('click', click, false);
  
  function click()
  {
    button.removeEventListener('click', click, false);
    document.body.removeChild(div);
    callback();
  }
  
  div.appendChild(button);
  div.id="popup";
  
  document.body.appendChild(div);
}

var fileio = {
  
  widgetCount: parseInt(widget.preferenceForKey('widgetCount')) || 1,
  
  removeMountPoint: function(name)
  {
    try {
      opera.io.filesystem.removeMountPoint(opera.io.filesystem.mountPoints.resolve(name));
    }
    catch(e)
    {
      opera.postError('Could not remove mountpoint: '+name+'\n\n'+e);
    }
  },
  
  browseForWidget: function()
  {
    opera.io.filesystem.browseForDirectory('widget'+fileio.widgetCount, 'downloads', fileio.browseForWidgetCallback, true);
  },
  
  browseForWidgetCallback: function(dir)
  {
    var file = null;
    var text = '';
    var xml = null;
    
    if (!dir || !dir.exists || !dir.isDirectory)
    {
      // die silently, user probably cancelled the operation
      fileio.removeMountPoint('widget'+fileio.widgetCount);
      return;
    }
    
    try {
      file = dir.resolve('config.xml');
      
      if (!file.exists)
      {
        messenger.alert("Widget could not be imported", "No config.xml file found.")
        throw new Error();
      }
      
      text = getFileContents(file);
      
      xml = (new DOMParser()).parseFromString(contents, "text/xml");
      
      if (!xml.getElementsByTagName('widget')[0])
      {
        messenger.alert("Widget could not be imported", "No <widget> element found in config.xml file.");
        throw new Error();
      }
    }
    catch(err)
    {
      fileio.removeMountPoint('widget'+fileio.widgetCount);
      return;
    }
    
    fileio.widgetCount++;
    widget.setPreferenceForKey(fileio.widgetCount, 'widgetCount');
    
    messenger.postMessage({
      command: "widgetAdded",
      config: text
    })
  },
  
  getFileContents: function(file)
  {
    var stream = file.open('', 'r');
    var contents = "";
    
    while (!stream.eof)
    {
      contents += (contents ? steam.newLine : '') + stream.readLine();
    }
    
    return contents;
  },
  
  parseXMLFile: function(file)
  {
    var stream = file.open('', 'r');
    var contents = "";
    
    while (!stream.eof)
    {
      contents += stream.readLine()+stream.newLine;
    }
    
    return (new DOMParser()).parseFromString(contents, "text/xml");
  }
  
};

function crawl()
{
  var storage = opera.io.filesystem.mountSystemDirectory('storage', 'storage');
  var application = opera.io.filesystem.mountSystemDirectory('application', 'application');

  var copyLocation = storage.resolve('emulator');
  
  if (copyLocation.exists)
  {
    if (!storage.deleteDirectory(copyLocation, true)) // delete it
    {
      opera.postError('copyLocation folder not deleted');
      return;
    }
  }
  copyLocation = storage.resolve('emulator');
  copyLocation = storage.createDirectory(copyLocation);
  
  copyFiles(application, copyLocation);
  
  document.body.style.backgroundColor = 'transparent';
  
  document.body.innerHTML = '<iframe src="'+copyLocation.path+'/index2.html"></iframe>';
}