
var pluginPanel = {
  
  STRINGS: {
    NO_DESCRIPTION: 'No description.',
    DEVICES_LIST: 'Devices using this plugin',
    NO_DEVICES: '(none)'
  },
  
  pluginTitleItems: {}, // plugin UI items references by their titles
  
  init: function()
  {
    for (var i in pluginHandler.plugins)
    {
      pluginPanel.addPluginUI(pluginHandler.plugins[i]);
    }
    
    g('plugins-add').addEventListener('click', fileio.browseForPlugin, false);
    
    g('plugins-delete').addEventListener('click', pluginPanel.onDeleteClick, false);
    
    g('plugins-deleteform').addEventListener('submit', pluginPanel.onDeleteConfirm, false);
    g('plugins-delete-cancel').addEventListener('click', pluginPanel.onDeleteCancel, false);
  },
  
  /**
   * Creates the HTML for a plugin item in the list.
   * Appends it to the document.
   *
   * @param plugin {Plugin} An object from pluginHandler.plugins
   */
  
  addPluginUI: function(plugin)
  {
    var html = null;
    var details = [];
    
    var attribution = [];
    
    if (plugin.version)
    {
      attribution.push(
        ['span',
          ['strong', 'Version: '],
          plugin.version
        ]
      );
    }
    
    if (plugin.author.name || plugin.author.organization || plugin.author.link)
    {
      var link =
        (plugin.author.link) ?
          ['a',
            'href', plugin.author.link,
            'data-tooltip', plugin.author.link
          ]
          :
          ['a']
      ;
      
      if (plugin.author.name)
      {
        link.push(['span', plugin.author.name]);
      }
      if (plugin.author.organization)
      {
        link.push(['span', plugin.author.organization]);
      }
      if (!plugin.author.name && !plugin.author.organisation)
      {
        link.push(['span', plugin.author.link]);
      }
      
      attribution.push(link);
    }
    
    if (attribution.length)
    {
      details.push(['p'].concat(attribution));
    }
    
    details.push(['p', plugin.description || pluginPanel.STRINGS.NO_DESCRIPTION]);
    
    details.push(
      ['div',
        'class', 'collapsable',
        ['h3', pluginPanel.STRINGS.DEVICES_LIST], // h3 is used as the title of the collapsable element
        ['ul']
      ]
    );
    
    html = createHTML(['dt',
      'data-pluginName', plugin.name,
      'onclick', pluginPanel.onTitleClick,
      ['span', plugin.name]
    ]);
    
    pluginPanel.pluginTitleItems[plugin.name] = html;
    
    g('plugins-list').appendChild(html);
    
    html = createHTML(['dd',
      details
    ]);
    
    g('plugins-list').appendChild(html);
  },
  
  /**
   * Event handler for clicking a title
   * Adds removes classNames for 'chosen' on 'dt' elements in the list
   */
  
  onTitleClick: function(evt)
  {
    var CLASSNAME = 'expanded';
    var item = evt.target;
    
    if (g('plugins-list').hasClass('deleting'))
    {
      // cannot switch devices when in delete mode
      return;
    }
    
    // get the 'dt' element, not a child of it
    while (item && 'dt' != item.tagName.toLowerCase())
    {
      item = item.parentNode;
    }
    if (item.hasClass(CLASSNAME))
    {
      item.removeClass(CLASSNAME);
      g('plugins-delete').disabled = true; // cannot delete when none selected 
    }
    else
    {
      var others = item.parentNode.getElementsByClassName(CLASSNAME);
      while (others[0])
      {
        others[0].removeClass(CLASSNAME); // live list
      }
      item.addClass(CLASSNAME);
      
      g('plugins-delete').disabled = false; // possible to delete a plugin now
    }
    
    var deviceList = item.nextSibling.getElementsByTagName('ul')[0];
    var pluginName = item.getAttribute('data-pluginName');
    
    // show the list of devices using this plugin
    pluginPanel.populateDeviceList(pluginName, deviceList);
    
    ui.resizeWindow();
  },
  
  onDeleteClick: function(evt)
  {
    var title = g('plugins-list').getElementsByClassName('expanded')[0];
    var deleteform = g('plugins-deleteform')
    
    if (!title) { return; } // nothing to delete
    
    var pluginName = title.getAttribute('data-pluginname');
    
    g('plugins-list').addClass('deleting');
    title.nextSibling.appendChild(deleteform);
    
    g('plugins-delete-name').textContent = pluginName;
    
    g('plugins-delete-cancel').focus(); // make easy for user to cancel operation    
    
    ui.resizeWindow();
    
    applyGlobalDisable(true, deleteform);
  },
  
  onDeleteCancel: function(evt)
  {
    g('plugins-list').removeClass('deleting');
    ui.resizeWindow();
    applyGlobalDisable(false);
  },
  
  onDeleteConfirm: function(evt)
  {
    var dt = g('plugins-list').getElementsByClassName('expanded')[0];
    var title = '';
    
    pluginPanel.onDeleteCancel();// undo the same stuff as cancelling (the disabled things etc.)
    evt.preventDefault();
    
    title = dt.getAttribute('data-pluginName');
    
    Control.deletePlugin(title);
  },
  
  /**
   * Removes the plugin UI.
   * Only the dt element is required, the dd sibling is implied and removed also.
   *
   * @param dt {Element} The dt element of the plugin ui
   */
  
  removePluginUI: function(title)
  {
    var dt = pluginPanel.pluginTitleItems[title];
    
    if (!dt)
    {
      opera.postError('Item not found: '+title);
      return;
    }
    
    // make sure the deleteform is not going to be removed
    g('plugins-list').parentNode.appendChild(g('plugins-deleteform'));
    
    var dd = dt.nextSibling;
    
    // remove dom elements
    dd.parentNode.removeChild(dd);
    dt.parentNode.removeChild(dt);
    
    delete pluginPanel.pluginTitleItems[title];
    
    g('plugins-delete').disabled = true; // cannot delete when none selected
  },
  
  /**
   * Populate's a list with a list of devices using the provided plugin name.
   *
   * @param pluginName {String} The name of the plugin
   * @param listEle {Element} The list element to populate
   */
  
  populateDeviceList: function(pluginName, listEle)
  {
    // clear the list
    while (listEle.firstChild)
    {
      listEle.removeChild(listEle.firstChild);
    }
    
    for (var i=0; i<Control.devices.length; i++)
    {
      if (Control.devices[i].plugins &&
          Control.devices[i].plugins.indexOf(pluginName)>=0)
      {
        listEle.appendChild(
          createHTML(
            ['li',
              Control.devices[i].title
            ]
          )
        )
      }
    }
    
    if ( 0 == listEle.getElementsByTagName('*').length)
    {
      listEle.appendChild(
        createHTML(
          ['li',
            pluginPanel.STRINGS.NO_DEVICES
          ]
        )
      )
    }
    
  }
  
  
};